function enh = enh_filter(sig, melrev, melenh, varargin)
%ENH_FILTER  Filter a time domain signal based on enhanced features.
%   XE = ENH_FILTER(X, M, ME) generates a dereverberated time domain
%   signal XE, based on a reverberant signal X, its mel-domain
%   spectral features M, and enhanced mel-spectral features ME.  The
%   features must be based on the filterbank used by the SIG2MEL
%   function.  If the number of columns in M or ME differ, the longer
%   matrix is truncated.  If FFT analysis of X results in more frames
%   than are in the features, X is also truncated.  If there are fewer
%   signal frames than feature frames, an error is signaled.
%
%   XE = ENH_FILTER(..., 'Option', Value, ...) can be used to adjust
%   the following parameters, with their default values indicated in
%   brackets:
%
%   'Window' [400]: Window size for the features, in samples.
%
%   'Step' [160]: Window step for the features, in samples.
%
%   'Filterbank' [generated by MELFBANK]: Custom filterbank matrix
%   that was used when extracting the features.

% Copyright (c) 2014, Heikki Kallasjoki, Kalle Palomäki, Ulpu Remes and Sami Keronen
% All rights reserved.
% See the included README.txt for full license terms.

%% input arguments

p = inputParser;
p.FunctionName = 'enh_filter';
addParamValue(p, 'Window', 400);
addParamValue(p, 'Step', 160);
addParamValue(p, 'Filterbank', []);
parse(p, varargin{:});
arg = p.Results;

%% generate FFT domain Wiener filter

dim = ceil((arg.Window + 1) / 2);

mframes = min(size(melenh, 2), size(melrev, 2));
if mframes < size(melenh, 2); melenh = melenh(:, 1:mframes); end
if mframes < size(melrev, 2); melrev = melrev(:, 1:mframes); end

if ~isempty(arg.Filterbank)
    mel2fft = arg.Filterbank;
else
    mel2fft = melfbank(dim, 'Channels', size(melrev, 1));
end
mel2fft = pinv(full(mel2fft));

w = min(max(mel2fft * melenh, 1e-9) ./ max(mel2fft * melrev, 1e-9), 1);
if mod(arg.Window, 2)
    w = [w; w(end:-1:2,:)];
else
    w = [w; w(end-1:-1:2,:)];
end

%% overlap-add resynthesize filtered signal

[frames, ~] = buffer(sig, arg.Window, arg.Window-arg.Step, 'nodelay');
nframes = size(frames, 2);

if nframes < mframes
    error('enh_filter:nframes', 'Unexpected number of frames', nframes, mframes);
elseif nframes > mframes;
    frames = frames(:,1:mframes);
    nframes = mframes;
end

enh = zeros((nframes - 1)*arg.Step + arg.Window, 1);
enhw = zeros(size(enh));

win = hamming(arg.Window);

enhframes = ifft(fft(frames) .* w, 'symmetric');

for f = 1:nframes
    t = (f-1)*arg.Step+1:(f-1)*arg.Step+arg.Window;
    enh(t) = enh(t) + enhframes(:,f) .* win;
    enhw(t) = enhw(t) + win.^2;
end

enhw(1:arg.Step) = enhw(arg.Step);
enhw(end-arg.Step+1:end) = enhw(end-arg.Step+1);

enh = enh ./ enhw;
