function [enh, estmask] = enh_missingdata(feats, model, varargin)
%ENH_MISSINGDATA  Missing data feature enhancement of reverberant speech.
%   This function implements the missing data feature enhancement of
%   Kallasjoki et al., "Recognition of reverberant speech by missing
%   data imputation and NMF feature enhancement", Proc. REVERB
%   Challenge, 2014.
%
%   ENH = ENH_MISSINGDATA(FEATS, MODEL) performs missing data mask
%   estimation (using MD_MASK) and BCMI reconstruction (using MD_BCMI)
%   on the input features FEATS, using the model MODEL generated by
%   ENH_MISSINGDATA_TRAIN.
%
%   ENH = ENH_MISSINGDATA(FEATS, MODEL, 'Option', Value, ...) can
%   be used to set the following parameters, with default values
%   given in brackets:
%
%   'MaskOptions' [{}]: A cell array of additional 'Key', Value pairs
%   for the mask estimation.  See the documentation of MD_MASK for a
%   list of available parameters.
%
%   [ENH, MASK] = ENH_MISSINGDATA(...) can be used to retrieve also
%   the estimated missing feature mask.

% Copyright (c) 2014, Heikki Kallasjoki, Kalle Palomäki, Ulpu Remes and Sami Keronen
% All rights reserved.
% See the included README.txt for full license terms.

%% input arguments

p = inputParser;
p.FunctionName = 'enh_missingdata';
addParamValue(p, 'MaskOptions', {});
parse(p, varargin{:});
arg = p.Results;

%% missing data feature enhancement

% estimate mask

mask = md_mask(feats, arg.MaskOptions{:});
if nargout > 1; estmask = mask; end

% input transformations: normalization, log domain, time context stacking

if model.meannorm > 0
    [feats, norm_vec] = meannorm(feats, model.meannorm, mask);
end
if model.log
    %feats = log(feats + model.epsilon) - log(model.epsilon) + 1;
    feats = feats.^0.3;
end
if model.T > 1
    feats = wstack(feats, model.T);
    mask = wstack(mask, model.T);
end

nwins = size(feats, 2);

% work around regions with no reliable features

bad = ~any(mask);

t = diff([0 bad 0]);
bad1 = find(t == 1);
bad2 = find(t == -1) - 1;
nbad = length(bad1);

for t = 1:nbad
    b = [bad1(t)-1 bad2(t)+1];
    if b(1) >= 1 && b(2) <= nwins
        fake = mask(:, b(1)) | mask(:, b(2));
    elseif b(2) <= nwins
        fake = mask(:, b(2));
    elseif b(1) >= 1
        fake = mask(:, b(1));
    else
        error('enh_missingdata:blank', 'Entire mask is blank!');
    end
    mask(:, bad1(t):bad2(t)) = repmat(fake, 1, b(2)-b(1)-1);
end

% reconstruct unreliable features

enh = md_bcmi(feats, mask, model.gmm);
%enh = struct('enh', enh, 'feats', feats, 'mask', mask, 'gmm', model.gmm);

% undo input transformations

if model.T > 1
    enh = wunstack(enh, model.T);
end
if model.log
    %enh = exp(enh + log(model.epsilon) - 1);
    enh = enh.^(1/0.3);
end
if model.meannorm > 0
    enh = bsxfun(@times, enh, norm_vec);
end
