function model = enh_missingdata_train(data, varargin)
%ENH_MISSINGDATA_TRAIN  Missing data model construction.
%   MODEL = ENH_MISSINGDATA_TRAIN(DATA) constructs a missing data
%   model for BCMI reconstruction with the ENH_MISSINGDATA function.
%   DATA is a cell array of training utterances, which typically
%   should be mel-domain linear magnitude spectrograms as returned by
%   SIG2MEL.
%
%   MODEL = ENH_MISSINGDATA_TRAIN(DATA, 'Option', Value, ...) can be
%   used to adjust the following parameters, with their default values
%   indicated in brackets:
%
%   'Normalize' [5]: If greater than zero, apply normalization with
%   the MEANNORM function, with this number as the N parameter.
%
%   'Log' [true]: If true, features are converted to the log domain.
%
%   'T' [1]: Length of time context for the model.
%
%   'Comps' [5]: Number of Gaussian components to use in the model.
%
%   'Epsilon' [1e-20]: Small adjustment added to the input features
%   in order to avoid negative infinities in the log domain.
%
%   'Verbose' [0]: Set to 1 for verbose GMM estimation.
%
%   'GMMBayes' [{}]: Cell array of extra options for gmmb_em.
%
%   This function requires the "GMMBayes" toolbox[1] to be in the path
%   for Gaussian mixture model estimation.
%   [1] http://www.it.lut.fi/project/gmmbayes/

% Copyright (c) 2014, Heikki Kallasjoki, Kalle Palomäki, Ulpu Remes and Sami Keronen
% All rights reserved.
% See the included README.txt for full license terms.

%% input arguments

p = inputParser;
p.FunctionName = 'enh_missingdata_train';
addParamValue(p, 'Normalize', 5);
addParamValue(p, 'Log', true);
addParamValue(p, 'T', 1);
addParamValue(p, 'Comps', 5);
addParamValue(p, 'Epsilon', 1e-20);
addParamValue(p, 'Verbose', false);
addParamValue(p, 'GMMBayes', {});
parse(p, varargin{:});
arg = p.Results;

%% train model

% prepare training data

dim = arg.T * size(data{1}, 1);
nwins = sum(cellfun(@(u) size(u, 2) - arg.T + 1, data));

tdata = zeros(dim, nwins);

at = 1;
for f = 1:length(data)
    utt = data{f};
    
    if arg.Normalize > 0; utt = meannorm(utt, arg.Normalize); end
    %if arg.Log > 0; utt = log(utt + arg.Epsilon) - log(arg.Epsilon) + 1; end
    if arg.Log > 0; utt = utt.^0.3; end
    if arg.T > 1; utt = wstack(utt, arg.T); end
    
    uttlen = size(utt, 2);
    tdata(:, at:at+uttlen-1) = utt;
    at = at + uttlen;
end

% construct the model

gmm = gmmb_em(tdata.', ...
              'components', arg.Comps, 'thr', 1e-6, 'verbose', arg.Verbose, ...
              arg.GMMBayes{:});

model = struct('gmm', gmm, ...
               'meannorm', arg.Normalize, 'log', arg.Log, 'T', arg.T, ...
               'epsilon', arg.Epsilon);
