function enh = enh_nmf(feats, model, varargin)
%ENH_NMF  NMF-based feature enhancement of reverberant speech.
%   This function implements the NMF-based feature enhancement of
%   Kallasjoki et al., "Recognition of reverberant speech by missing
%   data imputation and NMF feature enhancement", Proc. REVERB
%   Challenge, 2014.
%
%   ENH = ENH_NMF(FEATS, MODEL) performs NMF-based feature enhancement
%   of the input features FEATS, using the model MODEL generated by
%   ENH_NMF_TRAIN.
%
%   ENH = ENH_NMF(FEATS, MODEL, 'Option', Value, ...) can be used to
%   set the following additional parameters, with their default values
%   indicated in brackets:
%
%   'Tf' [20]: Length of reverberation filter in frames.
%
%   'Lambda' [1]: Sparsity coefficient parameter.
%
%   'ActFilter' [[1 -0.9 -0.8 -0.7]]: Activation matrix filter.
%
%   'OffWeight' [1]: Initial filter coefficient for indirect sound.
%
%   'Verbose' [0]: Set to true to print out statistics.
%
%   'MD' [{}]: Cell array of extra options for ENH_MISSINGDATA.
%
%   'Steps': Custom specification of A/R update order and iteration
%   counts. See the source code for details.

% Copyright (c) 2014, Heikki Kallasjoki, Kalle Palomäki, Ulpu Remes and Sami Keronen
% All rights reserved.
% See the included README.txt for full license terms.

%% input arguments

p = inputParser;
p.FunctionName = 'enh_nmf';
addParamValue(p, 'Tf', 20);
addParamValue(p, 'Lambda', 1);
addParamValue(p, 'ActFilter', [1 -0.9 -0.8 -0.7]);
addParamValue(p, 'OffWeight', 0.1);
addParamValue(p, 'MD', {});
addParamValue(p, 'Steps', {});
parse(p, varargin{:});
arg = p.Results;

%% derived parameter values

steps = arg.Steps;
if isempty(steps)
    steps = { 'A', 'bcmi', 50, arg.Lambda; ...
              'init-R', 0, 0, 0; ...
              'R', 'feats', 200, arg.Lambda; ...
              'A', 'feats', 100, arg.Lambda };
end
nsteps = size(steps, 1);

T = model.T;
D = model.dim;
if size(feats, 1) ~= D
    error('enh_nmf:dim', 'Feature dimensionality does not match model');
end

L = arg.Tf;
Tr = T + L - 1;

%% generate missing data imputation results for initial filter estimate

bcmifeats = enh_missingdata(feats, model.bcmi, arg.MD{:});

%% prepare for the NMF feature enhancement

% Mean-normalize features, apply frequency weighting, stack

nframes = size(feats, 2);

if model.meannorm > 0
    [feats, norm_vec] = meannorm(feats, model.meannorm);
    bcmifeats = bsxfun(@rdivide, bcmifeats, norm_vec);
end

feats = bsxfun(@times, feats, 1./model.freqweight);
bcmifeats = bsxfun(@times, bcmifeats, 1./model.freqweight);
S = bsxfun(@times, model.dict, repmat(1./model.freqweight, T, 1));

X = wstack(feats, Tr);
W = size(X, 2);

% L2-normalize basis

S = bsxfun(@rdivide, S, sqrt(sum(S.^2, 1)));
N = size(S, 2);

% From now on, work in single precision

feats = single(feats);
bcmifeats = single(bcmifeats);
S = single(S);

% Initial activation vector, filter matrix, and reconstructions

A = ones(N, W, 'single');

reverb = [ones(D, 1, 'single') 0.001*ones(D, L-1, 'single')];
reverb = reverb / (sum(reverb(:)) / D);

R = zeros(Tr*D, T*D, 'single');
for t = 1:T*D
    R(t:D:t+(L-1)*D, t) = reverb(1+mod(t-1,D), :).';
end

%% NMF iteration

OO = ones(Tr*D, W, 'single');

for step = 1:nsteps
    if strcmp(steps{step,1}, 'init-R')
        reverb = [ones(D, 1, 'single') arg.OffWeight*ones(D, L-1, 'single')];
        reverb = reverb / (sum(reverb(:)) / D);
        R = zeros(Tr*D, T*D, 'single');
        for t = 1:T*D
            R(t:D:t+(L-1)*D, t) = reverb(1+mod(t-1,D), :).';
        end
        
        % activation matrix filtering
        A = max(filter(arg.ActFilter, 1, A, [], 2), 1e-20);
        
        continue;
    end
    
    switch steps{step,2}
      case 'feats'
        target = feats;
      case 'bcmi'
        target = bcmifeats;
    end
    
    lambda = steps{step,4};
    SA = S*A;
    OO_SAt = OO * SA.';
    RS = R*S;
    RSt_OO_lambda = RS.' * OO + lambda;
    
    for it = 1:steps{step,3}
        fprintf(1, 'step %d/%d, iter %d/%d\n', step, nsteps, it, steps{step,3});
        
        switch steps{step,1}
          case 'R'
            recon = wunstack(R*SA, Tr, 1);
            R = R .* (wstack(target ./ recon, Tr) * SA.') ./ OO_SAt;
            
            reverb = zeros(D, L, 'single');
            for t = 1:T*D
                reverb(1+mod(t-1,D), :) = reverb(1+mod(t-1,D), :) + R(t:D:t+(L-1)*D, t).';
            end
            for t = 2:L
                tt = reverb(:,t-1) < reverb(:,t);
                reverb(tt,t) = reverb(tt,t-1);
            end
            reverb = reverb / (sum(reverb(:)) / D);
            for t = 1:T*D
                R(t:D:t+(L-1)*D, t) = reverb(1+mod(t-1,D), :).';
            end
            
          case 'A'
            recon = wunstack(RS*A, Tr, 1);
            A = A .* (RS.' * wstack(target ./ recon, Tr)) ./ RSt_OO_lambda;
            A(A < 1e-15) = 0;
        end
        
        %figure(1); t = 200:400; cr = wunstack(S*A, T, 1);
        %imagesc(log([target(:,t); recon(:,t); cr(:,t)])); title(['it = ' num2str(step) '/' num2str(it)]);
        %figure(2);
        %imagesc(reverb); title(['it = ' num2str(step) '/' num2str(it)]);
        %drawnow;
    end
end

clean = wunstack(S*A, T, 1);
clean = clean(:, 1:end-T);
t = size(clean, 2);

h = clean ./ recon(:,1:t);

% it's possible for the filter to become equal to 0, because the
% reverbed dictionary is longer than the clean dictionary.  this
% causes problems (log-mel features go to -Inf), so let's try to
% avoid that here.
hfloor = min(h(h>0));
h(h==0) = hfloor/10;

enh = bsxfun(@times, feats(:,1:t) .* h, model.freqweight);

if model.meannorm > 0
    enh = bsxfun(@times, enh, norm_vec);
end
