function model = enh_nmf_train(data, varargin)
%ENH_NMF_TRAIN  NMF model construction.
%   MODEL = ENH_NMF_TRAIN(DATA) constructs a NMF dictionary (and a
%   missing data model for bootstrapping).  DATA is a cell array of
%   training utterances, which typically should be mel-domain linear
%   magnitude spectrograms as returned by SIG2MEL.  If all training
%   utterances do not fit in memory simultaneously, DATA may also be a
%   struct with fields 'N', 'dim' and 'load', the latter of which must
%   be a function that returns a training utterance of dimensionality
%   'dim' when called with a single argument from 1 to N.
%
%   MODEL = ENH_MISSINGDATA_TRAIN(DATA, 'Option', Value, ...) can be
%   used to adjust the following parameters, with their default values
%   indicated in brackets:
%
%   'N' [min(10000, length(DATA))]: Target size for the dictionary.
%
%   'T' [10]: Length of clean speech NMF exemplars in frames.
%
%   'Normalize' [5]: If greater than zero, apply normalization with
%   the MEANNORM function, with this number as the N parameter.
%
%   'MDData' [DATA]: Set of features for missing data GMM model
%   construction.  Must be set if DATA has the function form.
%
%   'MD' [{}]: Extra options passed on to ENH_MISSINGDATA_TRAIN.
%
%   'Verbose' [0]: Set to 1 for verbose operation.
%
%   This function requires the "GMMBayes" toolbox[1] to be in the path
%   for Gaussian mixture model estimation.
%   [1] http://www.it.lut.fi/project/gmmbayes/

% Copyright (c) 2014, Heikki Kallasjoki, Kalle Palomäki, Ulpu Remes and Sami Keronen
% All rights reserved.
% See the included README.txt for full license terms.

%% input arguments

p = inputParser;
p.FunctionName = 'enh_nmf_train';
addParamValue(p, 'N', 10000);
addParamValue(p, 'T', 1);
addParamValue(p, 'Normalize', 5);
addParamValue(p, 'MDData', []);
addParamValue(p, 'MD', {});
addParamValue(p, 'Verbose', 0);
parse(p, varargin{:});
arg = p.Results;

if iscell(data)
    ndata = length(data);
    dim = size(data{1}, 1);
elseif isstruct(data)
    ndata = data.N;
    dim = data.dim;
    if isempty(arg.MDData)
        error('enh_nmf_train:mddata', 'MDData option required for function-based training data');
    end
else
    error('enh_nmf_train:data', 'Invalid training data format');
end

mddata = arg.MDData;
if isempty(mddata); mddata = data; end

%% train missing data model

bcmi = enh_missingdata_train(mddata, ...
                             'Normalize', arg.Normalize, 'Verbose', arg.Verbose, ...
                             arg.MD{:});

%% train NMF model

% select NMF exemplars uniformly from all files

uttex = repmat(floor(arg.N/ndata), ndata, 1);
while sum(uttex) < arg.N
    t = min(arg.N - sum(uttex), ndata);
    uttex(1:t) = uttex(1:t) + 1;
end

% extract dictionary

dict = zeros(arg.T * dim, arg.N);
freqweight = zeros(dim, 1);

at = 1;
for f = 1:ndata
    if arg.Verbose
        fprintf(1, 'extracting %d exemplars from utterance %d/%d\n', uttex(f), f, ndata);
    end
    
    if iscell(data)
        utt = data{f};
    else
        utt = data.load(f);
    end
    
    if arg.Normalize > 0
        utt = meannorm(utt, arg.Normalize);
    end
    
    freqweight = freqweight + sum(utt, 2);
    
    uttframes = size(utt, 2) - arg.T + 1;
    if uttex(f) >= uttframes
        expos = randperm(uttframes, uttex(f));
    else
        expos = randi(uttframes, 1, uttex(f));
    end
    
    for e = 1:uttex(f)
        t = expos(e);
        dict(:,at) = reshape(utt(:, t:t+arg.T-1), [], 1);
        at = at + 1;
    end
end

freqweight = freqweight/sum(freqweight);

model = struct('bcmi', bcmi, ...
               'dict', dict, 'freqweight', freqweight, ...
               'dim', dim, 'T', arg.T, 'meannorm', arg.Normalize);
