function reconstructed_feas1 = md_bcmi(feas1, mask1, gmm_model1)
%MD_BCMI  Bounded Conditional Mean Imputation algorithm.
%   RECON = MD_BCMI(FEATS, MASK, MODEL) implements the BCMI imputation
%   algorithm from Remes, "Bounded conditional mean imputation with an
%   approximate posterior", Proc. Interspeech, 2013.

% Copyright (c) 2014, Heikki Kallasjoki, Kalle Palomäki, Ulpu Remes and Sami Keronen
% All rights reserved.
% See the included README.txt for full license terms.

[feadim1, N1]=size(feas1);
[modeldim1, M1]=size(gmm_model1.mu); 

assert(modeldim1==feadim1);

% initialise variables

pos1=zeros(1,M1);
reconstructed1=zeros(feadim1,M1);
uncertainty1=zeros(feadim1,M1);
reconstructed_feas1=zeros(feadim1,N1);
reconstructed_uct1=zeros(feadim1,N1);
small_number=exp(-700);

sigma_tmp=cell(1,M1);
for z=1:M1
    sigma_tmp{z}=gmm_model1.sigma(:,:,z);
end

for n=1:N1
    
   %
   
   unreliable_feas1 = find(mask1(:,n)==0);
   reliable_feas1 = find(mask1(:,n)==1);
    
   %  
   
   if length(reliable_feas1)==feadim1
       reconstructed_feas1(:,n)=feas1(:,n);
       reconstructed_uct1(:,n)=zeros(feadim1,1);
       continue;
   end
   
   eye1 = eye(feadim1);
   
   if length(unreliable_feas1) < feadim1 %if need to reorganise

        k2 = length(reliable_feas1);
        
        for k1 = 1:length(unreliable_feas1)
            
            if reliable_feas1(k2) > unreliable_feas1(k1)

                eye1(unreliable_feas1(k1),unreliable_feas1(k1)) = 0;
                eye1(unreliable_feas1(k1),reliable_feas1(k2))=1;
                eye1(reliable_feas1(k2),unreliable_feas1(k1))=1; 
                eye1(reliable_feas1(k2),reliable_feas1(k2))=0;

                k2=k2-1;

                if k2<1
                    break
                end
            end
        end

    end % reorganisation
   
    [row,col]=find(eye1==1); % Matrix multiplication with eye1's on both sides equals
                             % picking the right rows and columns, see
                             % lines 75 and 76
% FEATURE RECONSTRUCTION

for m = 1:M1 

    if length(unreliable_feas1) < feadim1
    
    reorganised_feature=eye1*feas1(:,n);
    
    % compute mean and cov for unreliable data
    
    temppi=sigma_tmp{m}(row,col);
    sigma1=temppi(col,row);
    
    %sigma1=eye1*sigma_tmp{m}*eye1;
    %sigma1 = single(eye1*gmm_model1.sigma(:,:,m)*eye1); % Single precision float gives 
                                                        %the same results
                                                        %with better speed
	%sigma1=gmm_model1.sigma(:,:,m);
    %sigma1 = eye1*sigma1*eye1;

    mu1 = eye1*gmm_model1.mu(:,m);
    %mu1 = eye1*mu1;

    % define sr1, su1, sru1 etc.

    num_reliable1 = length(reliable_feas1);
    
    sr1 = sigma1(1:num_reliable1,1:num_reliable1);
    sm1 = sigma1(1:num_reliable1,num_reliable1+1:feadim1);
    sm2 = sigma1(num_reliable1+1:feadim1,1:num_reliable1);
    su1 = sigma1(num_reliable1+1:feadim1,num_reliable1+1:feadim1);

    mr2 = mu1(1:num_reliable1);
    mu2 = mu1(num_reliable1+1:feadim1);

    % marginal distribution for the unreliable components
    inv_sr1=sm2/sr1;      % Inv is faster to compute than pseudo-inv
                          % and in addition sm2*inv(sr1) can be substituted
                          % with sm2/sr1 which is even faster
    %inv_sr1=sm2*pinv(sr1,m);
    
    su3=diag(su1)-sum(inv_sr1.*sm1',2); % We can skip the computation of non-diagonal
                                        % elements since they're not used
    %su3 = su1-inv_sr1*sm1;
    
    mu3 = mu2+inv_sr1*(reorganised_feature(1:num_reliable1)-mr2);
    
    %%%% feature estimate
    
    reorganised_feature(num_reliable1+1:feadim1)=mu3;
    %reconstructed1(:,m) = eye1*reorganised_feature;
    reconstructed1(:,m) = reorganised_feature(row);
    %uncertainty1(:,m)=eye1*[zeros(num_reliable1,1); max(diag(su3),0)]; % See the comment on line 107
    uncertainty1(:,m)=eye1*[zeros(num_reliable1,1); max(su3,0)];
    
    %%%% cluster posterior
    pos1(m)=gmm_model1.weight(m)*mvnpdf(reorganised_feature(1:num_reliable1),mr2,sr1)+small_number;
    
    else
    
    reconstructed1(:,m) = gmm_model1.mu(:,m);
    uncertainty1(:,m) = diag(gmm_model1.sigma(:,:,m));
   
    pos1(m)=gmm_model1.weight(m);
    end
    
end

reconstructed_feas1(:,n)=sum(bsxfun(@times,reconstructed1,pos1/sum(pos1)),2);
reconstructed_uct1(:,n)=sum(bsxfun(@times,uncertainty1+power(bsxfun(@minus,reconstructed1,reconstructed_feas1(:,n)),2),pos1/sum(pos1)),2);

end

%%%% truncate posterior

lbm = -1*reconstructed_feas1./max(sqrt(reconstructed_uct1),small_number);
ubm = (feas1-reconstructed_feas1)./max(sqrt(reconstructed_uct1),small_number);
denominator1 = max(normcdf(ubm)-normcdf(lbm),small_number);
normlbm = normpdf(lbm)./denominator1;
normubm = normpdf(ubm)./denominator1;

reconstructed_feas1 = reconstructed_feas1+(normlbm-normubm).*sqrt(reconstructed_uct1);
