function mask = md_mask(feats, varargin)
%MD_MASK  Estimate a missing data mask for dereverberation.
%   MASK = MD_MASK(FEATS) estimates a missing data mask MASK suitable
%   for dereverberation of input features FEATS, which are assumed to
%   be mel-domain linear (not log) magnitude spectrum values.
%
%   MASK = MD_MASK(FEATS, 'Option', Value, ...) can be used to set
%   the following options, with default values given in brackets:
%
%   'Alpha' [19.0]: Alpha parameter for threshold sigmoid function.
%   'Beta' [0.43]: Beta parameter for threshold sigmoid function.
%   'Gamma' [1.4]: Gamma parameter for threshold sigmoid function.
%
%   'TC' [20]: AGC filter size, in frames.
%
%   The mask estimation algorithm is based on Palomäki et al.,
%   "Recognition of reverberant speech using full cepstral features
%   and spectral missing data", Proc. ICASSP, 2006.

% Copyright (c) 2014, Heikki Kallasjoki, Kalle Palomäki, Ulpu Remes and Sami Keronen
% All rights reserved.
% See the included README.txt for full license terms.

%% input arguments

p = inputParser;
p.FunctionName = 'md_mask';
addParamValue(p, 'Alpha', 19.0);
addParamValue(p, 'Beta', 0.43);
addParamValue(p, 'Gamma', 1.4);
addParamValue(p, 'TC', 20);
parse(p, varargin{:});
arg = p.Results;

%% load the mask modulation filter

persistent f1;
if isempty(f1)
    t = load('md_mask_coeffs.mat');
    f1 = t.f1;
end

[nchans, nframes] = size(feats);

feats = feats.^0.3; % psychoacoustics

% for threshold estimation, try to ignore leading/trailing silence
ssize = 20;
sene = smooth(sum(feats, 1), ssize);
eth = min(sene) + 0.1*(max(sene)-min(sene));
ss = max(1, find(sene > eth, 1, 'first') - ssize);
se = min(nframes, find(sene > eth, 1, 'last') + ssize);

mean_per_max = mean(feats(:,ss:se), 2) ./ max(feats(:,ss:se), [], 2);
blurredness = mean(mean_per_max);
threshold = sigmoidf(blurredness, arg.Alpha, arg.Beta, arg.Gamma);

mask = modfilteringagc(feats, f1, threshold, arg.TC);

function sig = sigmoidf(x, alpha, beta, gamma)
%SIGMOIDF  Parametric sigmoid function.
%   SIG = SIGMOIDF(X, ALPHA, BETA, GAMMA) computes the sigmoid
%   function gamma/(1+exp(-alpha*(x-beta))).  X may be a scalar,
%   vector or a matrix.

sig=gamma./(1+exp(-alpha.*(x-beta)));

function mask = modfilteringagc(feats, coeffs, threshold, timeconst)
%MODFILTERINGAGC  Filter function for the reverb mask estimator.

[nchans, nframes] = size(feats);

% Repeat last frame for a while and add some silence.

burnin = 50;
burnout = 20;
feats = [repmat(feats(:,1), 1, burnin) feats repmat(feats(:,end), 1, burnout) zeros(nchans, 200)];

delay = grpdelay(coeffs);
delay = round(delay(5));

hp = zeros(nchans, nframes);
for i = 1:nchans
    t = filter(coeffs, 1, feats(i,:));
    hp(i,:) = t(burnin+delay-2:burnin+delay-2+nframes-1);
end

hp = agc(hp, timeconst);
hp = bsxfun(@minus, hp, min(hp, [], 2));

ene = mean(hp, 2);
mask = bsxfun(@gt, hp, threshold * ene);

function feats_agc = agc(feats, sumfac)
%AGC  Automatic gain control function for the reverb mask estimator.

[nchans, nframes] = size(feats);

coeffs = [1:sumfac fliplr(1:sumfac)];
coeffs = coeffs/sqrt(sum(coeffs.^2));

fudge = 50; % for a kick start of filter

rateforfil = [feats(:, 1:fudge) feats feats(:,end-fudge:end) zeros(nchans, sumfac-fudge)];
rateforfil = abs(rateforfil);

fac = filter(coeffs, 1, rateforfil, [], 2);
fac = fac(:, sumfac+fudge+1:sumfac+fudge+nframes);

feats_agc = feats ./ fac;
